#include <cstdio>//for stderr,stdout
#include <cstdlib>//for atoi
#include <sys/stat.h> //for getting file attributes
#include <cstring> //for str operations

// a general matrix style structure
template <typename T>
struct matrix{
  int x;
  int y;
  T** data;
};

//function to cleanup our generic matrix structure
template <typename T>
void cleanup(matrix<T> &m){//using a reference to avoid copying the data
  for(int i=0;i<m.x;i++)
    delete [] m.data[i];
  delete [] m.data;
}

//get the filesize of afile
size_t fsize(const char* fname){
  struct stat st ;
  stat(fname,&st);
  return st.st_size;
}

//find out if a file exists
int fexists(const char* str){
  struct stat buffer ;
  return (stat(str, &buffer )==0 );
}

// a nice wrapper for getting files
FILE *getFILE(const char*fname,const char* mode){
  int writeFile = 0;
  for(size_t i=0;i<strlen(mode);i++)
    if(mode[i]=='w')
      writeFile = 1;
  if(writeFile&&fexists(fname)){//DRAGON
    fprintf(stderr,"\t-> File exists: %s exiting...\n",fname);
    exit(0);
  }
  FILE *fp;
  if(NULL==(fp=fopen(fname,mode))){
    fprintf(stderr,"\t->Error opening FILE handle for file:%s exiting\n",fname);
    exit(0);
  }
  return fp;
}



matrix<double> readFile(char *fname,int nInd){
  fprintf(stderr,"[%s] fname=%s\tnind=%d\n",__FUNCTION__,fname,nInd);
  FILE *fp = getFILE(fname,"r");
  size_t filesize =fsize(fname);
  if((filesize %(sizeof(double)*(2*nInd+1)) )){
    fprintf(stderr,"\n\t-> Possible error,binaryfiles might be broken\n");
    exit(-1);
  }
  int nSites = filesize/(sizeof(double)*(2*nInd+1));
  fprintf(stderr,"\t-> File contains: %d sites\n",nSites);
  double **data = new double*[nSites];
  for(int i=0;i<nSites;i++){
    double *tmp = new double[2*nInd+1];
    fread(tmp,sizeof(double),2*nInd+1,fp);
    data[i]= tmp;
  }
  fclose(fp);
  //make return type
  matrix<double> ret;
  ret.x = nSites;
  ret.y = 2*nInd+1;
  ret.data = data;
  return ret;
}

void writematrix(matrix<double> &m,FILE *fp){//this is a reference, to avoid copying data
  for(int i=0;i<m.x;i++){
    for(int j=0;j<m.y;j++)
      fprintf(fp,"%f\t",m.data[i][j]);
    fprintf(fp,"\n");
  }
}


int main(int argc,char **argv){
  if(argc==1){
    fprintf(stderr,"\t-> This programs eats .sfs files generated from realSFS\n");
    fprintf(stderr,"\t-> Supply filename of .sfs AND the number of individuals\n");
    fprintf(stderr,"\t-> This program can read different .sfs files\n");
    return 0;
  }
  fprintf(stderr,"\t-> Number of args given:%d\n",argc-1);//we don't count the progname
  if((argc-1) %2 ){
    fprintf(stderr,"\t-> Number of args given should be even (not odd)\n");
    return 0;
  }
  int nPop = (argc -1) /2;
  fprintf(stderr,"\t-> So we should have: %d populations\n",nPop);
  matrix<double> *pops = new matrix<double>[nPop];
  for(int i=0;i<nPop;i++)
    pops[i] = readFile(argv[2*i+1],atoi(argv[2*i+2]));
  fprintf(stderr,)
  
  writematrix(pops[0],stdout);
  //cleanup
  for(int i=0;i<nPop;i++)
    cleanup(pops[i]);

  delete [] pops;
}
